; PDF Page Extract configuration
; ©2020-1 Daniel Beardsmore
; https://telcontar.net/Software/PDF_Page_Extract


getConfigDef() {
	; This function in particular needs to do various things that the insane syntax won't allow
	
	configDef := []
	configDef["Last"] := { "_stupid_": "S"
		,	"pages": "S"
		,	"document": "S"
		,	"output folder": "S"
		,	"title": "S"
		,	"_stupid_": "S" }
	
	return configDef
}


getConfig() {
	; Read the user’s configuration
	; This can live either in the same directory as the application, or in Appdata\Telcontar
	
	configPath := getConfigPath()
	if (!configPath)
		return {}
	
	configDef := getConfigDef()
	
	config := []
	config["environment"] := readINIsection(configPath, "Environment", configDef["Environment"])
	config["last"] := readINIsection(configPath, "Last", configDef["Last"])
	
	config["authors"] := Array()
	Loop, 20 {
		IniRead, author, %configPath%, Authors, %A_Index%, %A_Space%
		if (author) {
			config["authors"].push(author)
		}
	}
	
	return config
}


writeConfig(config) {
	; Read the variable configuration
	
	configPath := getConfigPath()
	if (!configPath)
		return {}
	
	configDef := getConfigDef()
	
	writeINIsection(configPath, "Last", configDef["Last"], config["last"])
	
	authorIndex := 1
	for k, v in config["authors"] {
		IniWrite, %v%, %configPath%, Authors, %authorIndex%
		authorIndex++
	}
}


getConfigPath() {
	; Find where the program’s configuration lives
	; Returns false if no INI file exists
	
	configName := k_APPNAME . ".ini"
	configPath := A_ScriptDir . "\" . configName
	
	if (FileExist(configPath))
		return configPath
	
	configPath := A_AppData . "\Telcontar\" . configName
	if (FileExist(configPath))
		return configPath
	
	return false
}


readINIsection(iniPath, section, fields) {
	dict := {}
	
	iniOut := ""
	for fieldName, fieldType in fields
	{
		IniRead, iniOut, %iniPath%, %section%, %fieldName%, %A_Space%
		if (iniOut)
		{
			if (fieldType == "B") {
				iniOut := getHumanBool(iniOut)
			}
			else if (fieldType == "E") {
				; enum type: trim and convert to uppercase for comparison
				iniOut := trim(iniOut)
				StringUpper, iniOut, iniOut
			}
			dict[fieldName] := iniOut
		}
	}
	
	return dict
}


writeINIsection(iniPath, section, fields, sectionData) {
	for fieldName, fieldType in fields
	{
		if (fieldName == "_stupid_")
			continue
		
		iniOut := sectionData[fieldName]
		if (fieldType == "B") {
			iniOut := iniOut ? "true" : "false"
		}
		IniWrite, %iniOut%, %iniPath%, %section%, %fieldName%
	}
}


getHumanBool(humanBool) {
	; Convert a human-readable boolean value to true/false -- this allows some flexibility in the INI file contents
	
	humanBool := trim(humanBool)
	StringUpper, humanBool, humanBool
	if (humanBool == "TRUE" || humanBool == "YES" || humanBool == "1" || humanBool == "AFFIRMATIVE, MASTER")
		return true
	
	return false
}