﻿; PDF Page Extract
; ©2020-1 Daniel Beardsmore
; https://telcontar.net/Software/PDF_Page_Extract

#NoEnv
#NoTrayIcon
#SingleInstance Force

SetWorkingDir %A_ScriptDir%  ; Ensures a consistent starting directory.

; Pretendoconstants
global k_APPNAME := "PDF Page Extract"
global k_APPVERSION := "1.0.2"
global k_COPYRIGHT := "©2020–1 Daniel Beardsmore"
;global k_WEBSITE := "https://telcontar.net/Software/7-ade/"

;@Ahk2Exe-SetName PDF Page Extract
;@Ahk2Exe-SetDescription Extract pages from PDFs using Ghostscript
;@Ahk2Exe-SetVersion 1.0.2
;@Ahk2Exe-SetCopyright ©2020–1 Daniel Beardsmore
;@Ahk2Exe-SetOrigFilename PDF Page Extract.exe
;@Ahk2Exe-SetMainIcon PDF Page Extract.ico

global k_GS_ARGS := "-sDEVICE=pdfwrite -dNOPAUSE -dBATCH -dSAFER"
global k_GS_REG_SEARCH_KEY := "HKEY_LOCAL_MACHINE\SOFTWARE\Artifex\GPL Ghostscript"

global k_EXIT_INITERROR := 1
global k_EXIT_INTERNALERROR := 2
global k_EXIT_INVOKEERROR := 3

;@Ahk2Exe-IgnoreBegin
#Warn
global g_debug := true
global k_REL := "Build\"
;@Ahk2Exe-IgnoreEnd
/*@Ahk2Exe-Keep
global g_debug := false
global k_REL := ""
*/

#Include <Config>

global g_config := getConfig()

global g_ghostscript

global v_inputFile
global v_pageList
global v_title
global v_author
global v_outputFolder

g_ghostscript := locateGhostscript()
if (!g_ghostscript && A_Is64bitOS) {
	if (A_PtrSize == 4) {
		; We are compiled 32-bit on a 64-bit OS, so check the 64-bit Registry view
		SetRegView, 64
	}
	else {
		; We are compiled 64-bit, but the user could have only the 32-bit Ghostscript installed
		SetRegView, 32
	}
	g_ghostscript := locateGhostscript()
	SetRegView, Default
}
if (!g_ghostscript) {
	die(1, "Unable to find an installation of Ghostscript to use.")
}

createWindow()

if (A_Args.Length() > 0) {
	if (InStr(A_Args[1], "D") && substr(A_Args[1], -3) = ".PDF") {
		acceptFilePath(A_Args[1])
	}
}

return ; required to keep the window open


GuiClose:
	ExitApp
return



createWindow() {
	Gui, +OwnDialogs
	Gui, Margin, 32, 16
	
	if A_IsCompiled
		Gui, Add, Picture, w96 h96, %A_ScriptFullPath%
	else
		Gui, Add, Picture, w96 h96, % k_APPNAME . ".ico"
	
	Gui, Font, s16
	Gui, Add, Text, ym, %k_APPNAME%
	Gui, Font, s10
	
	Gui, Add, Text, y+2, Version %k_APPVERSION%
	Gui, Add, Text, y+2, %k_COPYRIGHT%
	Gui, Add, Text, y+2, % format("Ghostscript {} {} found", g_ghostscript["version"], g_ghostscript["exe"])
	; Gui, Add, Link, y+2, <a href="%k_WEBSITE%">%k_WEBSITE%</a>
	
	Gui Add, Edit, hidden section, Dummy
	
	Gui, Add, Text,     hp xm section, Source document:
	Gui, Add, Text,     hp y+5, Output folder:
	Gui, Add, Text,     hp y+5, Pages:
	Gui, Add, Text,     hp y+5, Output title:
	Gui, Add, Text,     hp y+5, Author:
	
	Gui, Add, Edit,     hp ys-3 section w450 readonly vv_inputFile, % g_config["last"]["document"]
	Gui, Add, Button,   hp x+10 gChooseFile, Choose ...
	Gui, Add, Edit,     hp xs w450 y+5 readonly vv_outputFolder, % g_config["last"]["output folder"]
	Gui, Add, Button,   hp x+10 gChooseFolder, Choose ...
	Gui, Add, Edit,     hp xs w75 y+5 vv_pageList, % g_config["last"]["pages"]
	Gui, Add, Edit,     hp y+5 xs w600 vv_title, % g_config["last"]["title"]
	; hp with a combo box applies incorrectly to the dropdown menu too!
	; also, capture the hwnd so that we can add items to it
	Gui, Add, ComboBox, y+5 xs w450 vv_author hwndhwnd_author
	
	Gui, Add, Button,   hp y+15 gextractPages default, Extract pages
	
	for k, v in g_config["authors"] {
		Control, Add, %v%, , ahk_id %hwnd_author%
	}

	GuiControl, Choose, v_author, 1
	
	Gui, Show, Center, %k_APPNAME%
}


GuiDropFiles(pWindow, pFiles, pControl, pX, pY) {
	if (pFiles.Length() == 0) {
		return
	}
	
	; Only take the first item passed in
	itemPath := pFiles[1]
	itemCheck := FileExist(itemPath)
	
	if (InStr(itemCheck, "D")) {
		; If a folder, set the new target directory
		acceptTargetDirectory(itemPath)
	}
	else if (itemCheck == "") {
		; Does not exist
	}
	else if (substr(itemPath, -3) = ".PDF") {
		acceptFilePath(itemPath)
	}
}


chooseFile() {
	FileSelectFile, newFile, 1, % g_config["last"]["document"], Select source document, Portable Document Format (*.pdf)
	if (newFile) {
		acceptFilePath(newFile)
	}
}


chooseFolder() {
	FileSelectFolder, outFolder, % "*" . g_config["last"]["output folder"], , Select output folder
	if (outFolder) {
		acceptTargetDirectory(outFolder)
	}
}


acceptFilePath(pPath) {
	SplitPath, pPath, newFileName, g_extractDir
	
	g_config["last"]["document"] := pPath
	GuiControl, , v_inputFile, % newFileName
}


acceptTargetDirectory(pDir) {
	g_config["last"]["output folder"] := pDir
	GuiControl, , v_outputFolder, % pDir
}


extractPages() {
	Gui, Submit, NoHide
	
	v_title_filename := StrReplace(v_title, """", "''")
	v_title_filename := StrReplace(v_title_filename, "/", "-")
	v_title_filename := StrReplace(v_title_filename, ":", "-")
	
	; Allow ranges such as 100-5 for 100–105, which Ghostscript appears not to recognise
	tmpPages := StrSplit(StrReplace(v_pageList, " ", ""), ",")
	tmpPageList := ""
	for k, v in tmpPages {
		if (InStr(v, "-")) {
			tmpRange := StrSplit(v, "-")
			if (strLen(tmpRange[2]) < strLen(tmpRange[1])) {
				tmpRange[2] := substr(tmpRange[1], 1, strLen(tmpRange[1]) - strLen(tmpRange[2])) . tmpRange[2]
				v := tmpRange[1] . "-" . tmpRange[2]
			}
		}
		
		if (tmpPageList != "")
			tmpPageList .= ","
		tmpPageList .= v
	}
	v_pageList := tmpPageList
	
	command := format("{} {} -sPageList={} -sOutputFile=""{}"" -f ""{}"" -c ""{}"""
	,	g_ghostscript["binary"]
	,	k_GS_ARGS
	,	v_pageList
	,	format("{}\{}.pdf", g_config["last"]["output folder"], v_title_filename)
	,	g_config["last"]["document"]
	,	format("[ /Title ({}) /Author ({}) /Creator ({} {}) /Subject () /Keywords () /DOCINFO pdfmark", v_title, v_author, k_APPNAME, k_APPVERSION))
	
	dbg(command)
	RunWait, %command%
	
	; Settings only captured during Gui, Submit
	g_config["last"]["pages"] := v_pageList
	g_config["last"]["title"] := v_title
	
	; Determine if we need to rearrange the authors array
	if (g_config["authors"].length() >= 1 && g_config["authors"][1] == v_author) {
		; Nothing needed
	}
	else {
		checkIndex := 2
		while (checkIndex <= g_config["authors"].length()) {
			if (g_config["authors"][checkIndex] == v_author || checkIndex > 19)
				g_config["authors"].removeAt(checkIndex)
			else
				checkIndex++
		}
		g_config["authors"].insertAt(1, v_author)
		
		GuiControl, , v_author, |
		GuiControlGet, hwnd_author, Hwnd, v_author
		for k, v in g_config["authors"] {
			Control, Add, %v%, , ahk_id %hwnd_author%
		}

		GuiControl, Choose, v_author, 1
	}
	
	writeConfig(g_config)
}


locateGhostscript() {
	gsExes := [ "gswin64c.exe", "gswin32c.exe" ]
	gsBinary := ""
	gsVer := ""
	
	Loop, Reg, %k_GS_REG_SEARCH_KEY%, K
	{
		gsVer := A_LoopRegName
		RegRead, gsDir, %k_GS_REG_SEARCH_KEY%\%A_LoopRegName%
		
		for i, exeName in gsExes {
			gsBinary := format("{}\bin\{}", gsDir, exeName)
			if (FileExist(gsBinary)) {
				return { version: gsVer, binary: gsBinary, exe: exeName }
			}
		}
	}
	
	return false
}


dbg(message) {
	; Don’t pollute someone else's WinDbg session with our messages; only report debugging messages if
	; we’re intentionally running in debugging mode
	
	if (g_debug)
		OutputDebug, %message%
}


die(exitCode, message) {
	msgBox 16, %k_APPNAME%, %message% 
	ExitApp, %exitCode%
}